/*
* This file is part of FD4: Four-Dimensional Distributed Dynamic Data structures
*  
* Copyright 2010-2013 Matthias Lieber, ZIH, TU Dresden, Germany
* matthias.lieber@tu-dresden.de
*  
* FD4 is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*  
* FD4 is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*  
* You should have received a copy of the GNU General Public License
* along with FD4.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sys/time.h>
#include <inttypes.h>
#include <stdlib.h>

/*#define FD4_NO_GETRUSAGE*/

#ifndef FD4_NO_GETRUSAGE
#include <sys/resource.h>
#endif


void gettimec(int64_t *);
void gettimec_(int64_t *);
void gettimec__(int64_t *);

/* return time in us (64bit integer) */
/* accuracy on ubuntu 10.04: 1 us */
void gettimec(int64_t *res)
{
        struct timeval v;
        gettimeofday(&v,NULL);
        *res=(int64_t)v.tv_usec + (1000*1000)*(int64_t)v.tv_sec;
}

void gettimec_(int64_t *res)
{
        struct timeval v;
        gettimeofday(&v,NULL);
        *res=(int64_t)v.tv_usec + (1000*1000)*(int64_t)v.tv_sec;
}

void gettimec__(int64_t *res)
{
        struct timeval v;
        gettimeofday(&v,NULL);
        *res=(int64_t)v.tv_usec + (1000*1000)*(int64_t)v.tv_sec;
}

#ifndef FD4_NO_GETRUSAGE

/* return time in us (64bit integer) */ 
/* accuracy on ubuntu 10.04: 10000 us */
void gettimec_rusage(int64_t *res)
{
        struct rusage r;
       	*res = 0;
        if( getrusage(RUSAGE_SELF,&r) != 0 ) return;
        *res = (int64_t)r.ru_utime.tv_usec + (1000*1000)*(int64_t)r.ru_utime.tv_sec
             + (int64_t)r.ru_stime.tv_usec + (1000*1000)*(int64_t)r.ru_stime.tv_sec;
}

void gettimec_rusage_(int64_t *res)
{
        struct rusage r;
       	*res = 0;
        if( getrusage(RUSAGE_SELF,&r) != 0 ) return;
        *res = (int64_t)r.ru_utime.tv_usec + (1000*1000)*(int64_t)r.ru_utime.tv_sec
             + (int64_t)r.ru_stime.tv_usec + (1000*1000)*(int64_t)r.ru_stime.tv_sec;
}

void gettimec_rusage__(int64_t *res)
{
        struct rusage r;
       	*res = 0;
        if( getrusage(RUSAGE_SELF,&r) != 0 ) return;
        *res = (int64_t)r.ru_utime.tv_usec + (1000*1000)*(int64_t)r.ru_utime.tv_sec
             + (int64_t)r.ru_stime.tv_usec + (1000*1000)*(int64_t)r.ru_stime.tv_sec;
}

#endif

#ifdef _TIMER_MAIN_
#include <stdio.h>
int main(void)
{
	int64_t t0=0,t1=0;
        int i;
        gettimec(&t0);
        printf("%ld\n",t0);


        gettimec(&t0);
        do gettimec(&t1);
        while(t1<=t0);
        printf("accuracy: %ld us\n",t1-t0);

	return 0;
}
#endif /* _TIMER_MAIN_ */

