/*
* This file is part of FD4: Four-Dimensional Distributed Dynamic Data structures
*  
* Copyright 2010-2013 Matthias Lieber, ZIH, TU Dresden, Germany
* matthias.lieber@tu-dresden.de
*  
* FD4 is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*  
* FD4 is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*  
* You should have received a copy of the GNU General Public License
* along with FD4.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
*   Fortran Wrapper for ParMETIS_V3_AdaptiveRepart which calls MPI_Comm_f2c. 
*/

#include <stdlib.h>
#include <stdio.h>
#include "parmetis.h"

#ifdef VTRACE
#include "vt_user.h"
#else
#define VT_IS_ON() 1
#define VT_USER_START(n)
#define VT_USER_END(n)
#define VT_ON()
#define VT_OFF()
#endif


#ifndef METIS_IDXTYPE
#define METIS_IDXTYPE idxtype
#endif

#define FRENAME(name0, name1, name2, name3, name4, dargs, cargs)   \
  void name1 dargs { name0 cargs; }                          \
  void name2 dargs { name0 cargs; }                          \
  void name3 dargs { name0 cargs; }                          \
  void name4 dargs { name0 cargs; }


void ParMETIS_F_V3_AdaptiveRepart(METIS_IDXTYPE *vtxdist, METIS_IDXTYPE *xadj, METIS_IDXTYPE *adjncy,
  METIS_IDXTYPE *vwgt, METIS_IDXTYPE *vsize, METIS_IDXTYPE *adjwgt, int *wgtflag, int *numflag,
  int *ncon, int *nparts, float *tpwgts, float *ubvec, float *ipc2redist,
  int *options, int *edgecut, METIS_IDXTYPE *part, int* comm_f)
{
  MPI_Comm comm_c;
  comm_c=MPI_Comm_f2c(*comm_f);
#ifndef VTRACE_PARMETIS
  int vtison;
#endif
  
  VT_USER_START("ParMETIS_V3_AdaptiveRepart");

#ifndef VTRACE_PARMETIS
  vtison = VT_IS_ON();
  if(vtison==1) VT_OFF();
#endif
  
  ParMETIS_V3_AdaptiveRepart(vtxdist, xadj, adjncy,
                             vwgt, vsize, adjwgt, wgtflag, numflag,
                             ncon, nparts, tpwgts,ubvec, ipc2redist,
                             options, edgecut, part,&comm_c);
  
#ifndef VTRACE_PARMETIS
  if(vtison==1) VT_ON();
#endif

  VT_USER_END("ParMETIS_V3_AdaptiveRepart");
}


FRENAME(ParMETIS_F_V3_AdaptiveRepart, 
        PARMETIS_F_V3_ADAPTIVEREPART,
	parmetis_f_v3_adaptiverepart,
	parmetis_f_v3_adaptiverepart_,
	parmetis_f_v3_adaptiverepart__,
	(METIS_IDXTYPE *vtxdist, METIS_IDXTYPE *xadj, METIS_IDXTYPE *adjncy, METIS_IDXTYPE *vwgt,
	 METIS_IDXTYPE *vsize, METIS_IDXTYPE *adjwgt, int *wgtflag, int *numflag, int *ncon,
	 int *nparts, float *tpwgts, float *ubvec, float *ipc2redist,
	 int *options, int *edgecut, METIS_IDXTYPE *part, int *comm_f),
	(vtxdist, xadj, adjncy, vwgt, vsize, adjwgt, wgtflag, numflag, ncon,
	 nparts, tpwgts, ubvec, ipc2redist, options, edgecut, part, comm_f)
) 

